
--[[
    TODO:
        - create access via fubar
        - create selectable list via fubar menu
    FUTURE:
        - Sort by methods
        - Replace small bag icon with drown down list
        - Add counter field to display: filtered items / total items
        - Enable / Disable commands
--]]

ZOOKEEPER_VERSION = 0.1
TYPE_MOUNTS = "MOUNT"
TYPE_COMPANIONS = "CRITTER"

ZooKeeper = LibStub("AceAddon-3.0"):NewAddon("ZooKeeper", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale("ZooKeeper", true)

function ZooKeeper:OnInitialize()
    local baseDB = LibStub("AceDB-3.0"):New("ZooKeeperDB")
    self.db = baseDB.global
    self:InitDB()
    self:SetupOptions()

    self:RegisterEvent("COMPANION_UPDATE", "CompanionUpdateHandler");
    self:RegisterEvent("COMPANION_LEARNED", "CompanionLearnedHandler");
end

function ZooKeeper:InitDB()
    local version = self.db.version
    if not version or version ~= ZOOKEEPER_VERSION then
        self.db.version = ZOOKEEPER_VERSION
    end
    if not self.db.columns then
        self.db.columns = 8
    end
    if not self.db.rows then
        self.db.rows = 10
    end
    if not self.db.lastViewed then
        self.db.lastViewed = TYPE_MOUNTS
    end
    if not self.db.buttonPadding then
        self.db.buttonPadding = 4
    end
    if not self.db.scale then
        self.db.scale = 1
    end
    if not self.db.opacity then
        self.db.opacity = 1
    end
    if not self.db.mountOrder then
        self.db.mountOrder = {}
    end
    if not self.db.companionOrder then
        self.db.companionOrder = {}
    end
end


function ZooKeeper:OnEnable()
    -- Called when the addon is enabled
    self.frame = self.Frame:New(L.Title, self.db, 'pets');

    if not self.button then
        self.button = CreateFrame("Button", "ZooKeeperOpenButton", PetPaperDollFrame, "ZooKeeperOpenButtonTemplate");
        self.button:ClearAllPoints();
        self.button:SetPoint("TOPRIGHT", PetPaperDollFrameCompanionFrame, "TOPRIGHT", -36, -36);
        self.button.frame = self.frame;
        self.button:SetScript("OnClick", function(self) self.frame:ToggleFrame() end);
        self:SecureHook("CharacterFrameTab_OnClick");
    end
end

function ZooKeeper:OnDisable()
    -- Called when the addon is disabled
end

function ZooKeeper:CompanionUpdateHandler()
    if self.frame then
        self.frame:UpdatePets()
    end
end

function ZooKeeper:CompanionLearnedHandler()
    self:Update()
end

function ZooKeeper:Update()
    if self.frame and self.frame:IsShown() then
        self.frame:UpdatePets()
    end
end

function ZooKeeper:CharacterFrameTab_OnClick (charFrame, button)
    local name = charFrame:GetName();

    if ( name == "CharacterFrameTab2" ) then
        if ( not self.frame:IsShown() ) then
            if (self.db.showWithPetPaperDoll and self.db.showWithPetPaperDoll == true) then
                self:Show()
            end
        end
    end
end

function ZooKeeper:Show()
    self.frame:Show()
end

--utility function: create a widget class
function ZooKeeper:NewClass(type, parentClass)
    local class = CreateFrame(type)
    class.mt = {__index = class}

    if parentClass then
        class = setmetatable(class, {__index = parentClass})
        class.super = parentClass
    end

    function class:Bind(o)
        return setmetatable(o, self.mt)
    end

    return class
end
